#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 16-Aug-2003
#################################################################################
#
# Poll - Script part of Etherea Realms designed to collect and talley poll results
# Copyright (C) 2000-2003 Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use eRealms::Database;						# Database handler
use eRealms::Param;						# Parameter control
use eRealms::Template;						# Template handler


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $param;							# Parameter handle
my $tmpl;							# Template handle

my $res;							# Results
my $statement;							# Query

my $exist;							# Does it exist
my $option;							# Option id
my $pixel = 4;							# Pixel per percent

my $name;							# Name
my $question;							# Question

my $percent;							# Percent of votes
my $spacer;							# Length of bar
my $votes;							# Total votes
my $options;							# Options

my %poll;							# Poll hash


#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;	
	$database = new eRealms::Database();

	# Connect and fetch
	$database->Connect();	


	# Parameter handling
	$param = new eRealms::Param($database, $cgi);
	$param->GetParam();
	
	# Link with hash
	$database->GetHashPoll(\%poll);

	# Instance
	$tmpl = new eRealms::Template(\%poll);


	# Document header
	print $cgi->header();
	$database->DocumentGetHeader();
	

	# Poll Identifier
	# Exists
	if (defined($param->{'PID'}))
	{
		# Does the almighty concure?
		($name, $question) = $database->DataGet("SELECT
			  PollName, 
			  PollQuestion
			FROM Poll 
			WHERE PollID=?",

		 $param->{'PID'});


		# You may pass
		if (defined($name))
		{
			# Cast your judgement
			if (defined($param->{'PRES'}))
			{
				# Jury deliberates
				($option) = $database->DataGet("SELECT OptionID
					FROM PollOption
					WHERE PollID=?
					  AND OptionName=?",

				  $param->{'PID'},
				  $param->{'PRES'});


				# Jury decides
				if (defined($option))
				{
					# Lousy theif
					($exist) = $database->DataGet("SELECT VoteHostname
						FROM PollVote
						WHERE PollID=?
						  AND VoteHostname=?",

					 $param->{'PID'},
					 $param->{'XADDR'});


					# Wrongly accused
					unless (defined($exist))
					{
						# Create vote
						$database->Quick("INSERT INTO PollVote
							(PollID,
							 VoteHostname,
							 OptionID)
							VALUES(?,?,?)",

						 $param->{'PID'},
						 $param->{'XADDR'},
						 $option
						);
					}
				}

				# Hung jury
				else
				{
					# Declare mis-trial
					$tmpl->Show('TmplMsg', MMESSAGE=>$poll{'MsgInvalid'});

					# Premature exit
					exit;
				}
			}


			# Give the boy a prize
			# Notice
			$tmpl->Show('TmplNotice', MNOTICE=>$poll{'MsgVote'}) if (defined($param->{'PRES'}));


			# Get total of all votes
			($votes) = $database->DataGet("SELECT COUNT(*)
				 FROM PollVote
				 WHERE PollID=?", $param->{'PID'});
			($options) = $database->DataGet("SELECT COUNT(*)
				FROM PollOption
				WHERE PollID=?", $param->{'PID'});


			# Tally up
			$votes = $votes - $options;

			# Avoid divide by zero
			$votes = ($votes > 0) ? $votes : 1;


			# Start display
			$database->Pull(\$statement, "SELECT
                                   COUNT(*)     AS \"Votes\",
                                   o.OptionName AS \"Name\"
                                FROM PollVote p, PollOption o
                                WHERE p.PollID=?
                                       AND p.PollID=o.PollID
                                       AND p.OptionID=o.OptionID
                                GROUP BY o.OptionId, o.OptionName
                                ORDER BY o.OptionId",

			 $param->{'PID'}
			);


			# Prep
			$tmpl->Show('TmplHeader', MTITLE=>$name, MQUESTION=>$question);

			# Loop and execute
			while ($res = $statement->fetchrow_hashref())
			{
				# Where are we
				$percent = int((($res->{'Votes'}-1)/$votes)*100);
				$spacer  = $percent * $pixel;

				# Display
				$tmpl->Show('TmplResult',
					MOPTION=>$res->{'Name'},
					MPERCENT=>$percent,
					MSPACER=>$spacer,
					MVOTES=>($res->{'Votes'}-1)
				 );
			}

			# Cleanup
			$tmpl->Show('TmplFooter', MTOTAL=>$votes);

			# End query and whining
			$statement->finish();

		}

		# Halt infidel!
		else
		{
			# Burn the stake!
			$tmpl->Show('TmplMsg', MMESSAGE=>$poll{'MsgMissing'})
		}
	}

	# Is missing
	else
	{
		# Signal error
		$tmpl->Show('TmplMsg', MMESSAGE=>$poll{'MsgMissing'})
	}

	# Document footer
	$database->DocumentGetFooter();
